﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Media;

using Infragistics.Controls.Grids;
using Infragistics.Controls.Grids.Primitives;

namespace ColumnWidth
{
	public partial class MainPage
	{
		public MainPage()
		{
			InitializeComponent();

			DataContext = this;
		}

		public DataSourceTableInformation Item1
		{
			get
			{
				return new DataSourceTableInformation("Categories", null, DataUtil.Categories);
			}
		}

		public DataSourceTableInformation Item2
		{
			get
			{
				return new DataSourceTableInformation( "Products", "CategoryID", DataUtil.Products );
			}
		}

		private void GetColumnWidths( object sender, RoutedEventArgs e )
		{
			unexpandedGroupByRows.Clear();
			StringBuilder = new StringBuilder();
			SaveIndex = tabControl.SelectedIndex;
			CurrentTabIndex = 0;
			tabControl.SelectedIndex = CurrentTabIndex;
			Dispatcher.BeginInvoke( SelectedIndexChanged );
		}

		private void SelectedIndexChanged()
		{
			ExpandAllGrids();
			Dispatcher.BeginInvoke( AllGridsExpanded1 );
		}

		private void AllGridsExpanded1()
		{
			Dispatcher.BeginInvoke( AllGridsExpanded2 );	// Even invoking the disatcher twice doesn't help.
		}

		private void AllGridsExpanded2()
		{
			DumpGrid();

			if ( ++CurrentTabIndex < tabControl.Items.Count )
			{
				tabControl.SelectedIndex = CurrentTabIndex;
				Dispatcher.BeginInvoke( SelectedIndexChanged );
			}

			else
			{
				tabControl.SelectedIndex = SaveIndex;
				Result.Text = StringBuilder.ToString().Trim();
				foreach ( GroupByRow groupByRow in unexpandedGroupByRows )
					groupByRow.IsExpanded = false;
			}
		}

		private StringBuilder StringBuilder { get; set; }
		private int SaveIndex { get; set; }
		private int CurrentTabIndex { get; set; }
		private readonly List<GroupByRow> unexpandedGroupByRows = new List<GroupByRow>();

		private void ExpandAllGrids()
		{
			foreach ( XamGrid xamGrid in FindAllXamGrids() )
				ExpandSingleGrid( xamGrid );
		}

		private void ExpandSingleGrid( XamGrid xamGrid )
		{
			ExpandViewableRecords( xamGrid.Rows );
		}

		private void ExpandViewableRecords( IEnumerable<Row> rows )
		{
			foreach ( Row row in rows )
			{
				switch ( row.RowType )
				{
					case RowType.DataRow:
						break;

					case RowType.GroupByRow:
						GroupByRow groupByRow = (GroupByRow) row;
						if ( !groupByRow.IsExpanded )
						{
							groupByRow.IsExpanded = true;
							unexpandedGroupByRows.Add( groupByRow );
						}
						ExpandViewableRecords( groupByRow.Rows );
						break;
				}
			}
		}

		private void DumpGrid()
		{
			foreach ( XamGrid xamGrid in FindAllXamGrids() )
			{
				DataSourceTableInformation dataSourceTableInformation = xamGrid.Tag as DataSourceTableInformation;
				if ( dataSourceTableInformation != null )
				{
					StringBuilder.AppendFormat( "Tab: {0}", dataSourceTableInformation.DisplayName );
					StringBuilder.AppendLine();
				}
				foreach ( Column column in xamGrid.Columns.AllVisibleColumns )
				{
					StringBuilder.AppendFormat( "Column {0}: ActualWidth {1}", column.Key, column.ActualWidth );
					if ( column.Width.HasValue )
					{
						Infragistics.Controls.Grids.ColumnWidth columnWidth = column.Width.Value;
						StringBuilder.AppendFormat( "; Width {0} {1}", columnWidth.Value, columnWidth.WidthType );
					}
					StringBuilder.AppendFormat( "; WidthResolved {0} {1}", column.WidthResolved.Value, column.WidthResolved.WidthType );
					StringBuilder.AppendLine();
				}
			}
		}

		private IEnumerable<XamGrid> FindAllXamGrids()
		{
			return FindVisualChildren<XamGrid>( this );
		}

		private static IEnumerable<T> FindVisualChildren<T>( DependencyObject dependencyObject ) where T : DependencyObject
		{
			if ( dependencyObject != null )
			{
				int count = VisualTreeHelper.GetChildrenCount( dependencyObject );
				for ( int i = 0; i < count; i++ )
				{
					DependencyObject child = VisualTreeHelper.GetChild( dependencyObject, i );
					if ( child != null && child is T )
					{
						yield return (T) child;
					}

					foreach ( T grandChild in FindVisualChildren<T>( child ) )
					{
						yield return grandChild;
					}
				}
			}
		}

		private void XamGridColumnResized( object sender, ColumnResizedEventArgs e )
		{
			if ( e.Columns.Count > 0 )
			{
				Column column = e.Columns[0];
				string result = column.Key + ": ActualWidth: " + column.ActualWidth;
				Infragistics.Controls.Grids.ColumnWidth columnWidth;
				if ( column.Width.HasValue )
				{
					columnWidth = column.Width.Value;
					result += " Width: " + columnWidth.Value + " " + columnWidth.WidthType;
				}
				columnWidth = column.WidthResolved;
				result += " WidthResolved: " + columnWidth.Value + " " + columnWidth.WidthType;
				Result.Text = result;
			}
		}

		private void XamGridLoaded( object sender, RoutedEventArgs e )
		{
			XamGrid grid = sender as XamGrid;
			if ( grid == null )
				return;

			DataSourceTableInformation dataSourceTableInformation = grid.Tag as DataSourceTableInformation;
			if ( dataSourceTableInformation == null )
				return;

			if ( String.IsNullOrEmpty( dataSourceTableInformation.GroupBy ) )
				return;

			Column column = FindColumn( grid, dataSourceTableInformation.GroupBy ) as Column;
			if ( column == null )
				return;

			column.IsGroupBy = true;
		}

		private static ColumnBase FindColumn( XamGrid grid, string fieldName )
		{
			ColumnBase columnBase = grid.Columns[fieldName];
			if ( columnBase != null )
				return columnBase;
			return (from c in grid.Columns
					where c.HeaderText == fieldName
					select c).FirstOrDefault();
		}
	}
}
